//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Laser Boy
//  Copyright (C) 2017 Carlos Sevila (@CarlosSevila)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

#include "entities.h"

const u8 kJumpValues[JUMP_SIZE] = {6, 6, 5, 5, 4, 4, 3, 3, 3, 2, 2, 2, 2, 2, 1, 1, 1, 1};

const TAnimFrame kAllAnimFrames[33] = {
	{ kPlayerIdleRight01, 8 },
	{ kPlayerIdleRight02, 8 },
	{ kPlayerIdleLeft01, 8 },
	{ kPlayerIdleLeft02, 8 },
	{ kPlayerIdleRight01, 4 },
	{ kPlayerIdleRight02, 4 },
	{ kPlayerIdleLeft01, 4 },
	{ kPlayerIdleLeft02, 4 },
	{ kPlayerJumpRight01, 4 },
	{ kPlayerJumpLeft01, 4 },
	{ kPlayerFallRight01, 4 },
	{ kPlayerFallLeft01, 4 },
	{ kChaser01, 4 },
	{ kChaser02, 4 },
	{ kChaser03, 4 },
	{ kChaser04, 4 },
	{ kDeath01, 4 },
	{ kDeath02, 4 },
	{ kDeath03, 4 },
	{ kDeath04, 4 },
	{ kTileBasicVoid, 4 },
	{ kChaserInmune01, 4 },
	{ kChaserInmune02, 4 },
	{ kChaserInmune03, 4 },
	{ kChaserInmune04, 4 },
	{ kBouncer01, 4 },
	{ kBouncer02, 4 },
	{ kBouncer03, 4 },
	{ kBouncer04, 4 },
	{ kAsteroid01, 8 },
	{ kAsteroid02, 8 },
	{ kAsteroid03, 8 },
	{ kAsteroid04, 8 }
};

#define AF kAllAnimFrames

TAnimFrame* const kPlayerIdleRight[3] = { &AF[0], &AF[1], 0 };
TAnimFrame* const kPlayerIdleLeft[3] = { &AF[2], &AF[3], 0 };
TAnimFrame* const kPlayerMoveRight[3] = { &AF[4], &AF[5], 0 };
TAnimFrame* const kPlayerMoveLeft[3] = { &AF[6], &AF[7], 0 };
TAnimFrame* const kPlayerJumpRight[2] = { &AF[8], 0 };
TAnimFrame* const kPlayerJumpLeft[2] = { &AF[9], 0 };
TAnimFrame* const kPlayerFallRight[2] = { &AF[10], 0 };
TAnimFrame* const kPlayerFallLeft[2] = { &AF[11], 0 };
TAnimFrame* const kEnemyChaser[5] = { &AF[12], &AF[13], &AF[14], &AF[15], 0 };
TAnimFrame* const kDeath[6] = { &AF[16], &AF[17], &AF[18], &AF[19], &AF[20], 0 };
TAnimFrame* const kEnemyChaserInmune[5] = { &AF[21], &AF[22], &AF[23], &AF[24], 0 };
TAnimFrame* const kEnemyBouncer[5] = { &AF[25], &AF[26], &AF[27], &AF[28], 0 };
TAnimFrame* const kEnemyAsteroid[5] = { &AF[29], &AF[30], &AF[31], &AF[32], 0 };

#undef AF

void DrawEntity(TEntity *entity) {
	u8 *pscr;

	pscr = cpct_getScreenPtr(CPCT_VMEM_START, entity->prevX, entity->prevY);
	cpct_drawSolidBox(pscr, cpct_px2byteM0(COLOR_BLACK, COLOR_BLACK), entity->width, entity->height);

	pscr = cpct_getScreenPtr(CPCT_VMEM_START, entity->x, entity->y);

	if (entity->animation.frameId == NO_ANIMATION_FRAME_ID) {
		// Code for no animation.
		cpct_drawSolidBox(pscr, cpct_px2byteM0(COLOR_BRIGHT_RED, COLOR_BRIGHT_RED), entity->width, entity->height);
	} else {
		if (UpdateAnimation(&entity->animation)) {
			cpct_drawSprite(entity->animation.frames[entity->animation.frameId]->sprite, pscr, entity->width, entity->height);
		}
	}
}

void EraseEntity(TEntity *entity) {
	u8 *pscr;

	pscr = cpct_getScreenPtr(CPCT_VMEM_START, entity->x, entity->y);
	cpct_drawSolidBox(pscr, cpct_px2byteM0(COLOR_BLACK, COLOR_BLACK), entity->width, entity->height);
}

void SetAnimationFrame(TEntity *entity, TAnimFrame **animFrame, u8 loop, u8 force) {
	if (entity->animation.frames != animFrame || force) {
		entity->animation.frames = animFrame;
		entity->animation.frameId = 0;
		entity->animation.time = 1;
		entity->animation.loop = loop;
	}
}

u8 CheckEntitiesCollisionBB(TEntity *entityA, TEntity *entityB) {
	return !(
		entityA->x > entityB->right
		|| entityB->x > entityA->right
		|| entityA->y > entityB->bottom
		|| entityB->y > entityA->bottom);
}
