//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Laser Boy
//  Copyright (C) 2017 Carlos Sevila (@CarlosSevila)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

#include <cpctelera.h>
#include "../entities/entities.h"
#include "../sprites/sprites.h"
#include "../input/input.h"
#include "../defines.h"
#include "../util/util.h"
#include "../text/text.h"
#include "../maps/maps.h"

#ifndef _GAME_H_
#define _GAME_H_

extern u8 initialMap;
extern u8 currentMap;
extern u8 seconds;
extern u16 minutes;
extern u16 totalDefeats;

void InitializeGame(void);
void GameLoop(void);
void CheckInput(u8 *ticks);

void UpdatePlayer(u8 *ticks);
void ResetPlayer(void);

void ChangeMap();
void LoadMap(u8 mapNumber);
u8* GetTile(u8 i);

u8 CheckMapCollision(u8 x, u8 y, u8 type);
u8 CheckMapCollisionBasic(u8 x, u8 y);
u8 CheckMapCollisionLaser(u8 x, u8 y);

void UpdateMovingEntity(u8 *ticks, TMovingEntity *movingEntity);
void CheckMovingEntity(TMovingEntity *movingEntity);
void ResetMovingEntities();
void SpawnEnemy(TMovingEntity *movingEntity, u8 x, u8 y, u8 type);

void SpawnDeath(u8 x, u8 y);
void CheckDeathDrawing(TEntity *entity);
void ResetDeaths();

void ActivateBlock(u8 x, u8 y);
void UpdateBlocks();
void ResetBlocks();

void DrawCurrentMap();
void UpdateTotalTime();
void DrawTotalTime(u8 forceDraw);
void DrawTotalDefeats();
void DrawCode();

u8 CheckDeath();
u8 CheckWin();

void CheckPlayerFade();
void CheckPause();

#endif
