//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Laser Boy
//  Copyright (C) 2017 Carlos Sevila (@CarlosSevila)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

#include "menu.h"

#include "../input/input.h"
#include "../text/text.h"
#include "../state/game.h"
#include "../audio/audio.h"

void InitializeMenu() {
    u8 *pscr;

    cpct_clearScreen_f64(0);

    // Vertical lines.
    // Left
    pscr = cpct_getScreenPtr(CPCT_VMEM_START, 0, 1);
    cpct_drawSolidBox(pscr, cpct_px2byteM0(COLOR_BRIGHT_BLUE, COLOR_BRIGHT_CYAN), 1, SCR_HEIGHT - 2);
    // Right
    pscr = cpct_getScreenPtr(CPCT_VMEM_START, SCR_WIDTH - 1, 1);
    cpct_drawSolidBox(pscr, cpct_px2byteM0(COLOR_BRIGHT_CYAN, COLOR_BRIGHT_BLUE), 1, SCR_HEIGHT - 2);

    // Horizontal lines.
    // Up
    pscr = cpct_getScreenPtr(CPCT_VMEM_START, 0, 0);
    cpct_drawSolidBox(pscr, cpct_px2byteM0(COLOR_BRIGHT_BLUE, COLOR_BRIGHT_BLUE), 40, 2);
    cpct_drawSolidBox(pscr + 40, cpct_px2byteM0(COLOR_BRIGHT_BLUE, COLOR_BRIGHT_BLUE), 40, 2);
    pscr = cpct_getScreenPtr(CPCT_VMEM_START, 1, 2);
    cpct_drawSolidBox(pscr, cpct_px2byteM0(COLOR_BRIGHT_CYAN, COLOR_BRIGHT_CYAN), 40, 2);
    cpct_drawSolidBox(pscr + 40, cpct_px2byteM0(COLOR_BRIGHT_CYAN, COLOR_BRIGHT_CYAN), 38, 2);
    // Down
    pscr = cpct_getScreenPtr(CPCT_VMEM_START, 1, SCR_HEIGHT - 4);
    cpct_drawSolidBox(pscr, cpct_px2byteM0(COLOR_BRIGHT_CYAN, COLOR_BRIGHT_CYAN), 40, 2);
    cpct_drawSolidBox(pscr + 40, cpct_px2byteM0(COLOR_BRIGHT_CYAN, COLOR_BRIGHT_CYAN), 38, 2);
    pscr = cpct_getScreenPtr(CPCT_VMEM_START, 0, SCR_HEIGHT - 2);
    cpct_drawSolidBox(pscr, cpct_px2byteM0(COLOR_BRIGHT_BLUE, COLOR_BRIGHT_BLUE), 40, 2);
    cpct_drawSolidBox(pscr + 40, cpct_px2byteM0(COLOR_BRIGHT_BLUE, COLOR_BRIGHT_BLUE), 40, 2);

    pscr = cpct_getScreenPtr(CPCT_VMEM_START, 32, 50);
    cpct_drawSprite(kLaserBoyLogo, pscr, 40, 80);

	DrawText("1 PLAY", 10, 80);
    DrawText("2 CODE", 10, 100);
	DrawText("3 KEYS", 10, 120);

	DrawText("2017 CARLOS SEVILA", 22, 170);

    cpct_waitVSYNC();
}

void MenuLoop() {
    while (1) {
        cpct_scanKeyboard_f();

        // Check menu options.
        if (cpct_isKeyPressed(Key_1)) {
			WaitKeyUp(Key_1);
            currentMap = 1;
            initialMap = currentMap;
            break;
        } else if (cpct_isKeyPressed(Key_2)) {
			WaitKeyUp(Key_2);

            // If the code is correct, change state.
            if (CodeOption()) {
                initialMap = currentMap;
                break;
            }
        } else if (cpct_isKeyPressed(Key_3)) {
			WaitKeyUp(Key_3);
            KeysOption();
        }
    }
}

u8 CodeOption() {
    u16 codeNumber;
    u8 *code[5];
    u8 *pscr;
    u8 i;

    // Draw text underline to complete.
    for (i = 0; i < 5; i++) {
        pscr = cpct_getScreenPtr(CPCT_VMEM_START, (i * FONT_WIDTH) + 24, FONT_HEIGHT - 2 + 100);
        cpct_drawSolidBox(pscr, cpct_px2byteM0(COLOR_BRIGHT_WHITE, COLOR_BRIGHT_WHITE), FONT_WIDTH - 1, 2);
    }

    // Read the code.
    for (i = 0; i < 5; i++) {
        code[i] = CheckKeyForCode(RedifineKey());
        DrawText(code[i], (i * FONT_WIDTH) + 24, 100);
    }

    // Clear de code text.
    pscr = cpct_getScreenPtr(CPCT_VMEM_START, 24, 100);
    cpct_drawSolidBox(pscr, cpct_px2byteM0(COLOR_BLACK, COLOR_BLACK), FONT_WIDTH * 5, FONT_HEIGHT);

    codeNumber = (*code[0] - 48) * 10000 + (*code[1] - 48) * 1000 + (*code[2] - 48) * 100 + (*code[3] - 48) * 10 + (*code[4] - 48);

    if (codeNumber == CODE_CHECKPOINT_1) {
        currentMap = 11;
        return 1;
    } else if (codeNumber == CODE_CHECKPOINT_2) {
        currentMap = 21;
        return 1;
    } else if (codeNumber == CODE_CHECKPOINT_3) {
        currentMap = 31;
        return 1;
    } else if (codeNumber == CODE_CHECKPOINT_4) {
        currentMap = 41;
        return 1;
    } else if (codeNumber == CODE_SECRET_MAP) {
        currentMap = 0;
        return 1;
    } else {
        return 0;
    }
}

void KeysOption() {
    // Up and down are not used finally.
    DrawText("LEFT", 24, 120);
    keys.left = RedifineKey();
    ClearText("LEFT", 24, 120);
    DrawText("RIGHT", 24, 120);
    keys.right = RedifineKey();
    ClearText("RIGHT", 24, 120);
    DrawText("FIRE", 24, 120);
    keys.fire = RedifineKey();
    ClearText("FIRE", 24, 120);
    DrawText("JUMP", 24, 120);
    keys.jump = RedifineKey();
    ClearText("JUMP", 24, 120);
    DrawText("PAUSE", 24, 120);
    keys.pause = RedifineKey();
    ClearText("PAUSE", 24, 120);
    DrawText("EXIT", 24, 120);
    keys.exit = RedifineKey();
    ClearText("EXIT", 24, 120);
}

u8* CheckKeyForCode(cpct_keyID key) {
    switch (key) {
        case Key_1: return "1";
        case Key_2: return "2";
        case Key_3: return "3";
        case Key_4: return "4";
        case Key_5: return "5";
        case Key_6: return "6";
        case Key_7: return "7";
        case Key_8: return "8";
        case Key_9: return "9";
        case Key_0: return "0";
    }

    return "X";
}
