//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Laser Boy
//  Copyright (C) 2017 Carlos Sevila (@CarlosSevila)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

#include "score.h"

#include <cpctelera.h>
#include "../defines.h"
#include "../sprites/sprites.h"
#include "../text/text.h"
#include "../input/input.h"

void InitializeScore(u8 *initialMap, u8 *currentMap, u16 *totalDefeats, u8 *seconds, u16 *minutes) {
    u8 *pscr;
    u8 digits;

    cpct_clearScreen_f64(0);

    // Vertical lines.
    // Left
    pscr = cpct_getScreenPtr(CPCT_VMEM_START, 0, 1);
    cpct_drawSolidBox(pscr, cpct_px2byteM0(COLOR_BRIGHT_BLUE, COLOR_BRIGHT_CYAN), 1, SCR_HEIGHT - 2);
    // Right
    pscr = cpct_getScreenPtr(CPCT_VMEM_START, SCR_WIDTH - 1, 1);
    cpct_drawSolidBox(pscr, cpct_px2byteM0(COLOR_BRIGHT_CYAN, COLOR_BRIGHT_BLUE), 1, SCR_HEIGHT - 2);

    // Horizontal lines.
    // Up
    pscr = cpct_getScreenPtr(CPCT_VMEM_START, 0, 0);
    cpct_drawSolidBox(pscr, cpct_px2byteM0(COLOR_BRIGHT_BLUE, COLOR_BRIGHT_BLUE), 40, 2);
    cpct_drawSolidBox(pscr + 40, cpct_px2byteM0(COLOR_BRIGHT_BLUE, COLOR_BRIGHT_BLUE), 40, 2);
    pscr = cpct_getScreenPtr(CPCT_VMEM_START, 1, 2);
    cpct_drawSolidBox(pscr, cpct_px2byteM0(COLOR_BRIGHT_CYAN, COLOR_BRIGHT_CYAN), 40, 2);
    cpct_drawSolidBox(pscr + 40, cpct_px2byteM0(COLOR_BRIGHT_CYAN, COLOR_BRIGHT_CYAN), 38, 2);
    // Down
    pscr = cpct_getScreenPtr(CPCT_VMEM_START, 1, SCR_HEIGHT - 4);
    cpct_drawSolidBox(pscr, cpct_px2byteM0(COLOR_BRIGHT_CYAN, COLOR_BRIGHT_CYAN), 40, 2);
    cpct_drawSolidBox(pscr + 40, cpct_px2byteM0(COLOR_BRIGHT_CYAN, COLOR_BRIGHT_CYAN), 38, 2);
    pscr = cpct_getScreenPtr(CPCT_VMEM_START, 0, SCR_HEIGHT - 2);
    cpct_drawSolidBox(pscr, cpct_px2byteM0(COLOR_BRIGHT_BLUE, COLOR_BRIGHT_BLUE), 40, 2);
    cpct_drawSolidBox(pscr + 40, cpct_px2byteM0(COLOR_BRIGHT_BLUE, COLOR_BRIGHT_BLUE), 40, 2);

    // Score.
    DrawText("SCORE", 36, 20);

    // Check completed.
    if (*currentMap == NUM_MAPS) {
        DrawText("LASER BOY COMPLETED", 22, 46);
        *currentMap = NUM_MAPS - 1;
    }

    // Maps.
    DrawText("FROM MAP:", 20, 70);
    DrawComposedNumber(*initialMap, 48, 70, 0);

    DrawText("TO MAP:", 20, 90);
    DrawComposedNumber(*currentMap, 48, 90, 0);

    // Defeats.
    DrawText("DEFEATS:", 20, 110);
    DrawComposedNumber(*totalDefeats, 48, 110, 0);

    // Time.
    DrawText("TIME:", 20, 130);

    digits = DrawComposedNumber(*minutes, 48, 130, 0);
    DrawText(":", 48 + digits * 2, 130);

    if (*seconds < 10) {
        DrawText("0", 48 + digits * 2 + 2, 130);

        digits = DrawComposedNumber(*seconds, 48 + digits * 2 + 4, 130, 0);
    } else {
        DrawComposedNumber(*seconds, 48 + digits * 2 + 2, 130, 0);
    }

    // Press fire.
    DrawText("PRESS FIRE", 30, 160);

    cpct_waitVSYNC();
}

void ScoreLoop() {
    u8 ticks = 0;
    u8 draw = 0;
    u8 *pscr;

    while (1) {
        cpct_scanKeyboard_f();

        if (cpct_isKeyPressed(keys.fire)) {
            WaitKeyUp(keys.fire);
            break;
        }

        cpct_waitVSYNC();

        if (ticks == 64) {
            ticks = 0;

            if (draw) {
                DrawText("PRESS FIRE", 30, 160);

                draw = 0;
            } else {
                pscr = cpct_getScreenPtr(CPCT_VMEM_START, 30, 160);
                cpct_drawSolidBox(pscr, cpct_px2byteM0(COLOR_BLACK, COLOR_BLACK), 22, 8);

                draw = 1;
            }
        }

        ticks++;
    }
}