//-----------------------------LICENSE NOTICE------------------------------------
//  This file is part of Laser Boy
//  Copyright (C) 2017 Carlos Sevila (@CarlosSevila)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

#include "text.h"

void DrawText(const u8 text[], u8 xPos, u8 yPos) {
    u8* pscr;
    u8 character;
    u8 x = 0;

    character = text[x];

    while (character != '\0') {
        pscr = cpct_getScreenPtr(CPCT_VMEM_START, (x * FONT_WIDTH) + xPos, yPos);

        if (character >= 48 && character <= 58) {
            // Numbers
            cpct_drawSprite(kNumbers[character - 48], pscr, FONT_WIDTH, FONT_HEIGHT);
        } else if (character != 32) { // 32 = Space
            // Letters
            cpct_drawSprite(kLetters[character - 65], pscr, FONT_WIDTH, FONT_HEIGHT);
        }

        character = text[++x];
    }
}

void ClearText(u8 text[], u8 xPos, u8 yPos) {
    u8* pscr;
    u8 character;
    u8 x = 0;

    character = text[x];

    while (character != '\0') {
        pscr = cpct_getScreenPtr(CPCT_VMEM_START, (x * FONT_WIDTH) + xPos, yPos);
        cpct_drawSolidBox(pscr, cpct_px2byteM0(COLOR_BLACK, COLOR_BLACK), FONT_WIDTH, FONT_HEIGHT);

        character = text[++x];
    }
}

u8 DrawComposedNumber(u16 number, u8 x, u8 y, u8 reverse) {
    const u8* digitChars[] = {"0", "1", "2", "3", "4", "5", "6", "7", "8", "9"};
    u8 digits[6];
    i8 i = 0;
    i8 size = 0;

    while (number > 9) {
        digits[i] = number % 10;
        number = number / 10;
        i++;
    }

    digits[i] = number;
    size = i;

    if (reverse) {
        for (i = 0; i <= size; i++) {
            DrawText(digitChars[digits[i]], x, y);
            x = x - 2;
        }
    } else {
        for (i = size; i > -1; i--) {
            DrawText(digitChars[digits[i]], x, y);
            x = x + 2;
        }
    }

    return size + 1;    // Number of digits of the number.
}